@@ -1,6 +1,6 @@
 /* This file is part of gPHPEdit, a GNOME2 PHP Editor.
  
-   Copyright  2003-2004 Andy Jeffries
+   Copyright (C) 2003-2005 Andy Jeffries
       andy@gphpedit.org
 	  
    For more information or to find the latest release, visit our 
@@ -123,6 +123,10 @@ void classbrowser_filelist_add(gchar *filename)
 {
 	ClassBrowserFile *file;
 
+	if (strstr(filename, "file://")==filename) {
+		filename += 6;
+	}
+
 	if (!classbrowser_filelist_find(filename) && is_php_file(filename)) {
 		file = g_malloc(sizeof(ClassBrowserFile));
 		file->filename = strip_double_slashes(filename);
@@ -171,6 +175,10 @@ void classbrowser_dirlist_add(gchar *dir)
 	struct dirent *dir_data;
 	GString *fullfilename;
 
+	if (strstr(dir, "file://")==dir) {
+		dir += 6;
+	}
+
 	dir_iter= opendir(dir);
 	if (dir_iter) {
 		errno=0;
@@ -196,7 +204,7 @@ void classbrowser_dirlist_add(gchar *dir)
 		//g_print("Err no: %d\n", errno);
 	}
 
-	dirlist = g_slist_append(dirlist, dir);
+	dirlist = g_slist_append(dirlist, g_strdup(dir));
 }
 
 
@@ -489,9 +497,9 @@ if (classbrowser_class_find_before_position(classname, &before)) {
 			gtk_tree_store_append (main_window.classtreestore, &iter, NULL);
 		}
 
-		gtk_tree_store_set (main_window.classtreestore, &iter,
-		                    NAME_COLUMN, class->classname, FILENAME_COLUMN, class->filename,
-		                    LINE_NUMBER_COLUMN, line_number, TYPE_COLUMN, CB_ITEM_TYPE_CLASS, ID_COLUMN, (gpointer)(class->identifierid), -1);
+		gtk_tree_store_set (GTK_TREE_STORE(main_window.classtreestore), &iter,
+		                    NAME_COLUMN, (class->classname), FILENAME_COLUMN, (class->filename),
+		                    LINE_NUMBER_COLUMN, line_number, TYPE_COLUMN, CB_ITEM_TYPE_CLASS, ID_COLUMN, (class->identifierid), -1);
 	}
 }
 
@@ -633,7 +641,6 @@ void classbrowser_update(void)
 	classbrowser_filelist_clear();
 	classbrowser_dirlist_clear();
 
-
 	// iterate open files
 	for(li = editors; li!= NULL; li = g_slist_next(li)) {
 		editor = li->data;
