@@ -1,6 +1,6 @@
 /* This file is part of gPHPEdit, a GNOME2 PHP Editor.
  
-   Copyright  2003-2004 Andy Jeffries
+   Copyright (C) 2003-2005 Andy Jeffries
       andy@gphpedit.org
 	  
    For more information or to find the latest release, visit our 
@@ -23,7 +23,8 @@
  
    The GNU General Public License is contained in the file COPYING.*/
 
-
+#include "config.h"
+#include <glib/gi18n.h>
 #include "find_replace.h"
 #include "main_window.h"
 
@@ -32,6 +33,10 @@ FindDialog find_dialog;
 ReplaceDialog replace_dialog;
 GtkWidget *replace_prompt_dialog;
 
+void find_destroy(GtkWidget *widget, gpointer data)
+{
+	gtk_widget_destroy(find_dialog.window1);
+}
 
 void find_destroyed(GtkWidget *widget, gpointer data)
 {
@@ -54,7 +59,7 @@ void find_clicked(GtkButton *button, gpointer data)
 	current_pos = gtk_scintilla_get_current_pos(GTK_SCINTILLA(main_window.current_editor->scintilla));
 
 	whole_document = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(find_dialog.radiobutton1));
-	text = gtk_editable_get_chars (GTK_EDITABLE(find_dialog.combo_entry1), 0, -1);
+	text = gtk_editable_get_chars (GTK_EDITABLE(find_dialog.entry1), 0, -1);
 
 	if (whole_document) {
 		current_pos = 0;
@@ -79,7 +84,7 @@ void find_clicked(GtkButton *button, gpointer data)
 
 	if (result == -1) {
 		// Show message saying could not be found.
-		gnome_dialog_run_and_close(GNOME_DIALOG(gnome_ok_dialog("Text not found.")));
+		gnome_dialog_run_and_close(GNOME_DIALOG(gnome_ok_dialog(_("Text not found."))));
 	}
 	else {
 		if (start_found == last_found) {
@@ -94,6 +99,18 @@ void find_clicked(GtkButton *button, gpointer data)
 	}
 }
 
+gint find_key_press_event(GtkWidget *widget, GdkEventKey *event,gpointer user_data)
+{
+	if (event->keyval == GDK_Return) {
+		find_clicked(NULL,NULL);
+		return TRUE;
+	}
+	else if (event->keyval == GDK_Escape) {
+		find_destroy(NULL, NULL);
+		return TRUE;
+	}
+	return FALSE;
+}
 
 void find_create(void)
 {
@@ -105,7 +122,7 @@ void find_create(void)
 	gtk_window_set_position (GTK_WINDOW(find_dialog.window1),GTK_WIN_POS_MOUSE);
 	gtk_window_set_transient_for(GTK_WINDOW(find_dialog.window1), GTK_WINDOW(main_window.window));
 	gtk_container_set_border_width (GTK_CONTAINER (find_dialog.window1), 6);
-	gtk_window_set_title (GTK_WINDOW (find_dialog.window1), "Find");
+	gtk_window_set_title (GTK_WINDOW (find_dialog.window1), _("Find"));
 	gtk_window_set_resizable (GTK_WINDOW (find_dialog.window1), FALSE);
 
 	find_dialog.vbox1 = gtk_vbox_new (FALSE, 0);
@@ -116,15 +133,16 @@ void find_create(void)
 	gtk_widget_show (find_dialog.frame1);
 	gtk_box_pack_start (GTK_BOX (find_dialog.vbox1), find_dialog.frame1, TRUE, TRUE, 3);
 
-	find_dialog.combo1 = gtk_combo_new ();
-	gtk_widget_show (find_dialog.combo1);
-	gtk_container_add (GTK_CONTAINER (find_dialog.frame1), find_dialog.combo1);
-	gtk_container_set_border_width (GTK_CONTAINER (find_dialog.combo1), 6);
+	find_dialog.alignment1 = gtk_alignment_new(0,0,1,1);
+	gtk_container_add (GTK_CONTAINER (find_dialog.frame1), find_dialog.alignment1);
+	gtk_alignment_set_padding(GTK_ALIGNMENT(find_dialog.alignment1), 6, 6, 6, 6);
+	gtk_widget_show(find_dialog.alignment1);
+	
+	find_dialog.entry1 = gtk_entry_new ();
+	gtk_widget_show (find_dialog.entry1);
+	gtk_container_add (GTK_CONTAINER (find_dialog.alignment1), find_dialog.entry1);
 
-	find_dialog.combo_entry1 = GTK_COMBO (find_dialog.combo1)->entry;
-	gtk_widget_show (find_dialog.combo_entry1);
-
-	find_dialog.label1 = gtk_label_new ("RegExp/String to search");
+	find_dialog.label1 = gtk_label_new (_("RegExp/String to search"));
 	gtk_widget_show (find_dialog.label1);
 	gtk_frame_set_label_widget (GTK_FRAME (find_dialog.frame1), find_dialog.label1);
 	gtk_label_set_justify (GTK_LABEL (find_dialog.label1), GTK_JUSTIFY_LEFT);
@@ -149,7 +167,7 @@ void find_create(void)
 	gtk_widget_show (find_dialog.vbox3);
 	gtk_container_add (GTK_CONTAINER (find_dialog.frame3), find_dialog.vbox3);
 
-	find_dialog.radiobutton1 = gtk_radio_button_new_with_mnemonic (NULL, "Whole document");
+	find_dialog.radiobutton1 = gtk_radio_button_new_with_mnemonic (NULL, _("Whole document"));
 	gtk_widget_show (find_dialog.radiobutton1);
 	gtk_box_pack_start (GTK_BOX (find_dialog.vbox3), find_dialog.radiobutton1, FALSE, FALSE, 0);
 	gtk_container_set_border_width (GTK_CONTAINER (find_dialog.radiobutton1), 2);
@@ -157,7 +175,7 @@ void find_create(void)
 	find_dialog.radiobutton1_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (find_dialog.radiobutton1));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (find_dialog.radiobutton1), TRUE);
 
-	find_dialog.radiobutton4 = gtk_radio_button_new_with_mnemonic (NULL, "From cursor");
+	find_dialog.radiobutton4 = gtk_radio_button_new_with_mnemonic (NULL, _("From cursor"));
 	gtk_widget_show (find_dialog.radiobutton4);
 	gtk_box_pack_start (GTK_BOX (find_dialog.vbox3), find_dialog.radiobutton4, FALSE, FALSE, 0);
 	gtk_container_set_border_width (GTK_CONTAINER (find_dialog.radiobutton4), 2);
@@ -165,7 +183,7 @@ void find_create(void)
 	find_dialog.radiobutton1_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (find_dialog.radiobutton4));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (find_dialog.radiobutton4), FALSE);
 
-	find_dialog.label3 = gtk_label_new ("Search");
+	find_dialog.label3 = gtk_label_new (_("Search"));
 	gtk_widget_show (find_dialog.label3);
 	gtk_frame_set_label_widget (GTK_FRAME (find_dialog.frame3), find_dialog.label3);
 	gtk_label_set_justify (GTK_LABEL (find_dialog.label3), GTK_JUSTIFY_LEFT);
@@ -178,21 +196,21 @@ void find_create(void)
 	gtk_widget_show (find_dialog.vbox4);
 	gtk_container_add (GTK_CONTAINER (find_dialog.frame4), find_dialog.vbox4);
 
-	find_dialog.radiobutton2 = gtk_radio_button_new_with_mnemonic (NULL, "Forwards");
+	find_dialog.radiobutton2 = gtk_radio_button_new_with_mnemonic (NULL, _("Forwards"));
 	gtk_widget_show (find_dialog.radiobutton2);
 	gtk_box_pack_start (GTK_BOX (find_dialog.vbox4), find_dialog.radiobutton2, FALSE, FALSE, 0);
 	gtk_container_set_border_width (GTK_CONTAINER (find_dialog.radiobutton2), 2);
 	gtk_radio_button_set_group (GTK_RADIO_BUTTON (find_dialog.radiobutton2), find_dialog.radiobutton2_group);
 	find_dialog.radiobutton2_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (find_dialog.radiobutton2));
 
-	find_dialog.radiobutton3 = gtk_radio_button_new_with_mnemonic (NULL, "Backwards");
+	find_dialog.radiobutton3 = gtk_radio_button_new_with_mnemonic (NULL, _("Backwards"));
 	gtk_widget_show (find_dialog.radiobutton3);
 	gtk_box_pack_start (GTK_BOX (find_dialog.vbox4), find_dialog.radiobutton3, FALSE, FALSE, 0);
 	gtk_container_set_border_width (GTK_CONTAINER (find_dialog.radiobutton3), 2);
 	gtk_radio_button_set_group (GTK_RADIO_BUTTON (find_dialog.radiobutton3), find_dialog.radiobutton2_group);
 	find_dialog.radiobutton2_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (find_dialog.radiobutton3));
 
-	find_dialog.label4 = gtk_label_new ("Direction");
+	find_dialog.label4 = gtk_label_new (_("Direction"));
 	gtk_widget_show (find_dialog.label4);
 	gtk_frame_set_label_widget (GTK_FRAME (find_dialog.frame4), find_dialog.label4);
 	gtk_label_set_justify (GTK_LABEL (find_dialog.label4), GTK_JUSTIFY_LEFT);*/
@@ -205,14 +223,14 @@ void find_create(void)
 	gtk_widget_show (find_dialog.vbox5);
 	gtk_container_add (GTK_CONTAINER (find_dialog.frame5), find_dialog.vbox5);
 
-	find_dialog.radiobutton5 = gtk_radio_button_new_with_mnemonic (NULL, "RegExp");
+	find_dialog.radiobutton5 = gtk_radio_button_new_with_mnemonic (NULL, _("RegExp"));
 	gtk_widget_show (find_dialog.radiobutton5);
 	gtk_box_pack_start (GTK_BOX (find_dialog.vbox5), find_dialog.radiobutton5, FALSE, FALSE, 0);
 	gtk_container_set_border_width (GTK_CONTAINER (find_dialog.radiobutton5), 2);
 	gtk_radio_button_set_group (GTK_RADIO_BUTTON (find_dialog.radiobutton5), find_dialog.radiobutton5_group);
 	find_dialog.radiobutton5_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (find_dialog.radiobutton5));
 
-	find_dialog.radiobutton6 = gtk_radio_button_new_with_mnemonic (NULL, "String");
+	find_dialog.radiobutton6 = gtk_radio_button_new_with_mnemonic (NULL, _("String"));
 	gtk_widget_show (find_dialog.radiobutton6);
 	gtk_box_pack_start (GTK_BOX (find_dialog.vbox5), find_dialog.radiobutton6, FALSE, FALSE, 0);
 	gtk_container_set_border_width (GTK_CONTAINER (find_dialog.radiobutton6), 2);
@@ -220,7 +238,7 @@ void find_create(void)
 	find_dialog.radiobutton5_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (find_dialog.radiobutton6));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (find_dialog.radiobutton6), TRUE);
 
-	find_dialog.label5 = gtk_label_new ("Type");
+	find_dialog.label5 = gtk_label_new (_("Type"));
 	gtk_widget_show (find_dialog.label5);
 	gtk_frame_set_label_widget (GTK_FRAME (find_dialog.frame5), find_dialog.label5);
 	gtk_label_set_justify (GTK_LABEL (find_dialog.label5), GTK_JUSTIFY_LEFT);
@@ -229,16 +247,16 @@ void find_create(void)
 	gtk_widget_show (find_dialog.hbox3);
 	gtk_box_pack_start (GTK_BOX (find_dialog.vbox2), find_dialog.hbox3, TRUE, TRUE, 3);
 
-	find_dialog.checkbutton1 = gtk_check_button_new_with_mnemonic ("Whole word");
+	find_dialog.checkbutton1 = gtk_check_button_new_with_mnemonic (_("Whole word"));
 	gtk_widget_show (find_dialog.checkbutton1);
 	gtk_box_pack_start (GTK_BOX (find_dialog.hbox3), find_dialog.checkbutton1, FALSE, FALSE, 4);
 
-	find_dialog.checkbutton2 = gtk_check_button_new_with_mnemonic ("Ignore case while searching");
+	find_dialog.checkbutton2 = gtk_check_button_new_with_mnemonic (_("Ignore case while searching"));
 	gtk_widget_show (find_dialog.checkbutton2);
 	gtk_box_pack_start (GTK_BOX (find_dialog.hbox3), find_dialog.checkbutton2, TRUE, FALSE, 0);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (find_dialog.checkbutton2), TRUE);
 
-	find_dialog.label2 = gtk_label_new ("Options");
+	find_dialog.label2 = gtk_label_new (_("Options"));
 	gtk_widget_show (find_dialog.label2);
 	gtk_frame_set_label_widget (GTK_FRAME (find_dialog.frame2), find_dialog.label2);
 	gtk_label_set_justify (GTK_LABEL (find_dialog.label2), GTK_JUSTIFY_LEFT);
@@ -262,9 +280,14 @@ void find_create(void)
 	gtk_signal_connect (GTK_OBJECT (find_dialog.button5),
 	                    "clicked", GTK_SIGNAL_FUNC (find_clicked), NULL);
 
+	gtk_signal_connect (GTK_OBJECT (find_dialog.window1), "key_press_event", GTK_SIGNAL_FUNC (find_key_press_event), NULL);
+
 	// Hide the dialog box when the user clicks the cancel_button
+	//gtk_signal_connect_object (GTK_OBJECT (find_dialog.button4),
+	//                           "clicked", GTK_SIGNAL_FUNC (gtk_widget_hide), (gpointer) find_dialog.window1);
+	// Don't hide it, destroy it, the same as closing it (for consistency) - AJ 2005-10-14
 	gtk_signal_connect_object (GTK_OBJECT (find_dialog.button4),
-	                           "clicked", GTK_SIGNAL_FUNC (gtk_widget_hide), (gpointer) find_dialog.window1);
+	                           "clicked", GTK_SIGNAL_FUNC (find_destroy), NULL);
 							   
 	gtk_signal_connect(GTK_OBJECT(find_dialog.window1), "destroy", GTK_SIGNAL_FUNC(find_destroyed), NULL);
 }
@@ -276,6 +299,12 @@ void replace_destroyed(GtkWidget *widget, gpointer data)
 	replace_dialog.window2=NULL;
 }
 
+void replace_destroy(GtkWidget *widget, gpointer data)
+{
+	gtk_widget_destroy(replace_dialog.window2);
+}
+
+
 
 void replace_prompt_reply(gint reply,gpointer data)
 {
@@ -335,7 +364,7 @@ void replace_clicked(GtkButton *button, gpointer data)
 
 	if (result == -1) {
 		// Show message saying could not be found.
-		gnome_dialog_run_and_close(GNOME_DIALOG(gnome_ok_dialog("Text not found.")));
+		gnome_dialog_run_and_close(GNOME_DIALOG(gnome_ok_dialog(_("Text not found."))));
 	}
 	else {
 		if (start_found == last_found) {
@@ -348,7 +377,7 @@ void replace_clicked(GtkButton *button, gpointer data)
 		gtk_scintilla_set_selection_end(GTK_SCINTILLA(main_window.current_editor->scintilla), end_found);
 		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(replace_dialog.checkbutton8))) {
 			// Prompt for replace?
-			replace_prompt_dialog = gnome_question_dialog("Do you want to replace this occurence?",
+			replace_prompt_dialog = gnome_question_dialog(_("Do you want to replace this occurence?"),
 			                        replace_prompt_reply, NULL);
 			gnome_dialog_run_and_close(GNOME_DIALOG(replace_prompt_dialog));
 		}
@@ -424,13 +453,13 @@ void replace_all_clicked(GtkButton *button, gpointer data)
 
 	message = g_string_new("");
 	if (numfound==0) {
-		g_string_sprintf(message, "%s not found, no replacements made.", text);
+		g_string_sprintf(message, _("%s not found, no replacements made."), text);
 	}
 	else if (numfound==1) {
-		g_string_sprintf(message, "1 occurence of %s found, replaced.", text);
+		g_string_sprintf(message, _("1 occurence of %s found, replaced."), text);
 	}
 	else {
-		g_string_sprintf(message, "%d occurences of %s found, all replaced.", numfound, text);
+		g_string_sprintf(message, _("%d occurences of %s found, all replaced."), numfound, text);
 	}
 
 	replace_all_dialog = gnome_ok_dialog(message->str);
@@ -440,6 +469,20 @@ void replace_all_clicked(GtkButton *button, gpointer data)
 	gtk_scintilla_goto_pos(GTK_SCINTILLA(main_window.current_editor->scintilla), start_pos);
 }
 
+
+gint replace_key_press_event(GtkWidget *widget, GdkEventKey *event,gpointer user_data)
+{
+	if (event->keyval == GDK_Return) {
+		replace_clicked(NULL,NULL);
+		return TRUE;
+	}
+	else if (event->keyval == GDK_Escape) {
+		replace_destroy(NULL, NULL);
+		return TRUE;
+	}
+	return FALSE;
+}
+
 void replace_create(void)
 {
 	replace_dialog.radiobutton17_group = NULL;
@@ -461,14 +504,15 @@ void replace_create(void)
 	gtk_widget_show (replace_dialog.frame10);
 	gtk_box_pack_start (GTK_BOX (replace_dialog.vbox10), replace_dialog.frame10, TRUE, TRUE, 0);
 
-	replace_dialog.combo2 = gtk_combo_new ();
-	gtk_widget_show (replace_dialog.combo2);
-	gtk_container_add (GTK_CONTAINER (replace_dialog.frame10), replace_dialog.combo2);
-	gtk_container_set_border_width (GTK_CONTAINER (replace_dialog.combo2), 6);
-
-	replace_dialog.entry1 = GTK_COMBO (replace_dialog.combo2)->entry;
+	replace_dialog.alignment1 = gtk_alignment_new(0,0,1,1);
+	gtk_container_add (GTK_CONTAINER (replace_dialog.frame10), replace_dialog.alignment1);
+	gtk_alignment_set_padding(GTK_ALIGNMENT(replace_dialog.alignment1), 6, 6, 6, 6);
+	gtk_widget_show(replace_dialog.alignment1);
+	
+	replace_dialog.entry1 = gtk_entry_new ();
 	gtk_widget_show (replace_dialog.entry1);
-
+	gtk_container_add (GTK_CONTAINER (replace_dialog.alignment1), replace_dialog.entry1);
+	
 	replace_dialog.label10 = gtk_label_new (_("RegExp/String to search"));
 	gtk_widget_show (replace_dialog.label10);
 	gtk_frame_set_label_widget (GTK_FRAME (replace_dialog.frame10), replace_dialog.label10);
@@ -478,13 +522,14 @@ void replace_create(void)
 	gtk_widget_show (replace_dialog.frame15);
 	gtk_box_pack_start (GTK_BOX (replace_dialog.vbox10), replace_dialog.frame15, TRUE, TRUE, 0);
 
-	replace_dialog.combo3 = gtk_combo_new ();
-	gtk_widget_show (replace_dialog.combo3);
-	gtk_container_add (GTK_CONTAINER (replace_dialog.frame15), replace_dialog.combo3);
-	gtk_container_set_border_width (GTK_CONTAINER (replace_dialog.combo3), 6);
-
-	replace_dialog.entry2 = GTK_COMBO (replace_dialog.combo3)->entry;
+	replace_dialog.alignment2 = gtk_alignment_new(0,0,1,1);
+	gtk_container_add (GTK_CONTAINER (replace_dialog.frame15), replace_dialog.alignment2);
+	gtk_alignment_set_padding(GTK_ALIGNMENT(replace_dialog.alignment2), 6, 6, 6, 6);
+	gtk_widget_show(replace_dialog.alignment2);
+	
+	replace_dialog.entry2 = gtk_entry_new ();
 	gtk_widget_show (replace_dialog.entry2);
+	gtk_container_add (GTK_CONTAINER (replace_dialog.alignment2), replace_dialog.entry2);
 
 	replace_dialog.label15 = gtk_label_new (_("String to Replace"));
 	gtk_widget_show (replace_dialog.label15);
@@ -625,7 +670,7 @@ void replace_create(void)
 	gtk_widget_show (replace_dialog.button8);
 	gtk_box_pack_start (GTK_BOX (replace_dialog.hbox8), replace_dialog.button8, TRUE, FALSE, 0);
 
-	replace_dialog.button9 = gtk_button_new_with_mnemonic("Replace _all");
+	replace_dialog.button9 = gtk_button_new_with_mnemonic(_("Replace _all"));
 	gtk_widget_show (replace_dialog.button9);
 	gtk_box_pack_start (GTK_BOX (replace_dialog.hbox8), replace_dialog.button9, TRUE, FALSE, 0);
 
@@ -635,9 +680,15 @@ void replace_create(void)
 	gtk_signal_connect (GTK_OBJECT (replace_dialog.button9),
 	                    "clicked", GTK_SIGNAL_FUNC (replace_all_clicked), NULL);
 
+	gtk_signal_connect (GTK_OBJECT (replace_dialog.window2), "key_press_event", GTK_SIGNAL_FUNC (replace_key_press_event), NULL);
+
 	// Hide the dialog box when the user clicks the cancel_button
+	//gtk_signal_connect_object (GTK_OBJECT (replace_dialog.button7),
+	//                           "clicked", GTK_SIGNAL_FUNC (gtk_widget_hide), (gpointer) replace_dialog.window2);
+	// Don't hide it, destroy it, the same as closing it (for consistency) - AJ 2005-10-14
 	gtk_signal_connect_object (GTK_OBJECT (replace_dialog.button7),
-	                           "clicked", GTK_SIGNAL_FUNC (gtk_widget_hide), (gpointer) replace_dialog.window2);
+	                           "clicked", GTK_SIGNAL_FUNC (replace_destroy), NULL);
+							   
 	gtk_signal_connect(GTK_OBJECT(replace_dialog.window2), "destroy",
 	                   GTK_SIGNAL_FUNC(replace_destroyed), NULL);
 }
