@@ -1,6 +1,6 @@
 /* This file is part of gPHPEdit, a GNOME2 PHP Editor.
  
-   Copyright  2003-2004 Andy Jeffries
+   Copyright (C) 2003-2005 Andy Jeffries
       andy@gphpedit.org
 	  
    For more information or to find the latest release, visit our 
@@ -164,15 +164,15 @@ static void main_window_create_toolbars(void)
 	gtk_toolbar_set_style (GTK_TOOLBAR (main_window.toolbar_main), GTK_TOOLBAR_ICONS);
 
 	// Add the File operations to the Main Toolbar
-	main_window.toolbar_main_button_new = gtk_toolbar_insert_stock (GTK_TOOLBAR (main_window.toolbar_main),GTK_STOCK_NEW, "New File", NULL, NULL, NULL, -1);
+	main_window.toolbar_main_button_new = gtk_toolbar_insert_stock (GTK_TOOLBAR (main_window.toolbar_main),GTK_STOCK_NEW, _("New File"), NULL, NULL, NULL, -1);
 	gtk_widget_show (main_window.toolbar_main_button_new);
-	main_window.toolbar_main_button_open = gtk_toolbar_insert_stock (GTK_TOOLBAR (main_window.toolbar_main),GTK_STOCK_OPEN, "Open File", NULL, NULL, NULL, -1);
+	main_window.toolbar_main_button_open = gtk_toolbar_insert_stock (GTK_TOOLBAR (main_window.toolbar_main),GTK_STOCK_OPEN, _("Open File"), NULL, NULL, NULL, -1);
 	gtk_widget_show (main_window.toolbar_main_button_open);
-	main_window.toolbar_main_button_save = gtk_toolbar_insert_stock (GTK_TOOLBAR (main_window.toolbar_main),GTK_STOCK_SAVE, "Save File", NULL, NULL, NULL, -1);
+	main_window.toolbar_main_button_save = gtk_toolbar_insert_stock (GTK_TOOLBAR (main_window.toolbar_main),GTK_STOCK_SAVE, _("Save File"), NULL, NULL, NULL, -1);
 	gtk_widget_show (main_window.toolbar_main_button_save);
-	main_window.toolbar_main_button_save_as = gtk_toolbar_insert_stock (GTK_TOOLBAR (main_window.toolbar_main),GTK_STOCK_SAVE_AS, "Save File As...", NULL, NULL, NULL, -1);
+	main_window.toolbar_main_button_save_as = gtk_toolbar_insert_stock (GTK_TOOLBAR (main_window.toolbar_main),GTK_STOCK_SAVE_AS, _("Save File As..."), NULL, NULL, NULL, -1);
 	gtk_widget_show (main_window.toolbar_main_button_save_as);
-	main_window.toolbar_main_button_close = gtk_toolbar_insert_stock (GTK_TOOLBAR (main_window.toolbar_main),GTK_STOCK_CLOSE, "Close File", NULL, NULL, NULL, -1);
+	main_window.toolbar_main_button_close = gtk_toolbar_insert_stock (GTK_TOOLBAR (main_window.toolbar_main),GTK_STOCK_CLOSE, _("Close File"), NULL, NULL, NULL, -1);
 	gtk_widget_show (main_window.toolbar_main_button_close);
 	gtk_signal_connect (GTK_OBJECT (main_window.toolbar_main_button_open), "clicked", GTK_SIGNAL_FUNC (on_open1_activate), NULL);
 	gtk_signal_connect (GTK_OBJECT (main_window.toolbar_main_button_new), "clicked", GTK_SIGNAL_FUNC (on_new1_activate), NULL);
@@ -182,31 +182,31 @@ static void main_window_create_toolbars(void)
 	gtk_toolbar_append_space(GTK_TOOLBAR(main_window.toolbar_main));
 
 	// Add the Undo operations to the Main Toolbar
-	main_window.toolbar_main_button_undo = gtk_toolbar_insert_stock (GTK_TOOLBAR (main_window.toolbar_main),GTK_STOCK_UNDO, "Undo last change", NULL, NULL, NULL, -1);
+	main_window.toolbar_main_button_undo = gtk_toolbar_insert_stock (GTK_TOOLBAR (main_window.toolbar_main),GTK_STOCK_UNDO, _("Undo last change"), NULL, NULL, NULL, -1);
 	gtk_widget_show (main_window.toolbar_main_button_undo);
-	main_window.toolbar_main_button_redo = gtk_toolbar_insert_stock (GTK_TOOLBAR (main_window.toolbar_main),GTK_STOCK_REDO, "Redo last change", NULL, NULL, NULL, -1);
+	main_window.toolbar_main_button_redo = gtk_toolbar_insert_stock (GTK_TOOLBAR (main_window.toolbar_main),GTK_STOCK_REDO, _("Redo last change"), NULL, NULL, NULL, -1);
 	gtk_widget_show (main_window.toolbar_main_button_redo);
 	gtk_signal_connect (GTK_OBJECT (main_window.toolbar_main_button_undo), "clicked", GTK_SIGNAL_FUNC (on_undo1_activate), NULL);
 	gtk_signal_connect (GTK_OBJECT (main_window.toolbar_main_button_redo), "clicked", GTK_SIGNAL_FUNC (on_redo1_activate), NULL);
 	gtk_toolbar_append_space(GTK_TOOLBAR(main_window.toolbar_main));
 
 	// Add the Clipboard operations to the Main Toolbar
-	main_window.toolbar_main_button_cut = gtk_toolbar_insert_stock (GTK_TOOLBAR (main_window.toolbar_main),GTK_STOCK_CUT, "Cut current selection", NULL, NULL, NULL, -1);
+	main_window.toolbar_main_button_cut = gtk_toolbar_insert_stock (GTK_TOOLBAR (main_window.toolbar_main),GTK_STOCK_CUT, _("Cut current selection"), NULL, NULL, NULL, -1);
 	gtk_signal_connect (GTK_OBJECT (main_window.toolbar_main_button_cut), "clicked", GTK_SIGNAL_FUNC (on_cut1_activate), NULL);
 	gtk_widget_show (main_window.toolbar_main_button_cut);
-	main_window.toolbar_main_button_copy = gtk_toolbar_insert_stock (GTK_TOOLBAR (main_window.toolbar_main),GTK_STOCK_COPY, "Copy current selection", NULL, NULL, NULL, -1);
+	main_window.toolbar_main_button_copy = gtk_toolbar_insert_stock (GTK_TOOLBAR (main_window.toolbar_main),GTK_STOCK_COPY, _("Copy current selection"), NULL, NULL, NULL, -1);
 	gtk_signal_connect (GTK_OBJECT (main_window.toolbar_main_button_copy), "clicked", GTK_SIGNAL_FUNC (on_copy1_activate), NULL);
 	gtk_widget_show (main_window.toolbar_main_button_save_as);
-	main_window.toolbar_main_button_paste = gtk_toolbar_insert_stock (GTK_TOOLBAR (main_window.toolbar_main),GTK_STOCK_PASTE, "Paste current selection", NULL, NULL, NULL, -1);
+	main_window.toolbar_main_button_paste = gtk_toolbar_insert_stock (GTK_TOOLBAR (main_window.toolbar_main),GTK_STOCK_PASTE, _("Paste current selection"), NULL, NULL, NULL, -1);
 	gtk_signal_connect (GTK_OBJECT (main_window.toolbar_main_button_paste), "clicked", GTK_SIGNAL_FUNC (on_paste1_activate), NULL);
 	gtk_widget_show (main_window.toolbar_main_button_save_as);
 	gtk_toolbar_append_space(GTK_TOOLBAR(main_window.toolbar_main));
 
 	// Add the Search operations to the Main Toolbar
-	main_window.toolbar_main_button_find = gtk_toolbar_insert_stock (GTK_TOOLBAR (main_window.toolbar_main),GTK_STOCK_FIND, "Find text", NULL, NULL, NULL, -1);
+	main_window.toolbar_main_button_find = gtk_toolbar_insert_stock (GTK_TOOLBAR (main_window.toolbar_main),GTK_STOCK_FIND, _("Find text"), NULL, NULL, NULL, -1);
 	gtk_widget_show (main_window.toolbar_main_button_find);
 	gtk_signal_connect (GTK_OBJECT (main_window.toolbar_main_button_find), "clicked", GTK_SIGNAL_FUNC (on_find1_activate), NULL);
-	main_window.toolbar_main_button_replace = gtk_toolbar_insert_stock (GTK_TOOLBAR (main_window.toolbar_main),GTK_STOCK_FIND_AND_REPLACE, "Find/Replace text", NULL, NULL, NULL, -1);
+	main_window.toolbar_main_button_replace = gtk_toolbar_insert_stock (GTK_TOOLBAR (main_window.toolbar_main),GTK_STOCK_FIND_AND_REPLACE, _("Find/Replace text"), NULL, NULL, NULL, -1);
 	gtk_signal_connect (GTK_OBJECT (main_window.toolbar_main_button_replace), "clicked", GTK_SIGNAL_FUNC (on_replace1_activate), NULL);
 	gtk_widget_show (main_window.toolbar_main_button_replace);
 
@@ -217,26 +217,26 @@ static void main_window_create_toolbars(void)
 	                       BONOBO_DOCK_ITEM_BEH_NORMAL, BONOBO_DOCK_TOP, 2, 0, 0);
 	gtk_container_set_border_width (GTK_CONTAINER (main_window.toolbar_find), 1);
 
-	main_window.toolbar_find_search_label = gtk_label_new("Search for: ");
+	main_window.toolbar_find_search_label = gtk_label_new(_("Search for: "));
 	gtk_widget_show(main_window.toolbar_find_search_label);
 	gtk_toolbar_append_widget(GTK_TOOLBAR (main_window.toolbar_find), main_window.toolbar_find_search_label,NULL,NULL);
 
 	main_window.toolbar_find_search_entry = gtk_entry_new();
 	gtk_widget_show(main_window.toolbar_find_search_entry);
-	gtk_toolbar_append_widget(GTK_TOOLBAR (main_window.toolbar_find), main_window.toolbar_find_search_entry,"Incremental search",NULL);
+	gtk_toolbar_append_widget(GTK_TOOLBAR (main_window.toolbar_find), main_window.toolbar_find_search_entry, _("Incremental search"),NULL);
 	gtk_signal_connect_after(GTK_OBJECT(main_window.toolbar_find_search_entry), "insert_text", GTK_SIGNAL_FUNC(inc_search_typed), NULL);
 	gtk_signal_connect_after(GTK_OBJECT(main_window.toolbar_find_search_entry), "key_release_event", GTK_SIGNAL_FUNC(inc_search_key_release_event), NULL);
 	gtk_signal_connect_after(GTK_OBJECT(main_window.toolbar_find_search_entry), "activate", GTK_SIGNAL_FUNC(inc_search_activate), NULL);
 	gtk_toolbar_append_space(GTK_TOOLBAR(main_window.toolbar_find));
 
-	main_window.toolbar_find_goto_label = gtk_label_new("Go to line: ");
+	main_window.toolbar_find_goto_label = gtk_label_new(_("Go to line: "));
 	gtk_widget_show(main_window.toolbar_find_goto_label);
 	gtk_toolbar_append_widget(GTK_TOOLBAR (main_window.toolbar_find), main_window.toolbar_find_goto_label,NULL,NULL);
 
 	main_window.toolbar_find_goto_entry = gtk_entry_new_with_max_length(6);
 	gtk_entry_set_width_chars(GTK_ENTRY(main_window.toolbar_find_goto_entry),7);
 	gtk_widget_show(main_window.toolbar_find_goto_entry);
-	gtk_toolbar_append_widget(GTK_TOOLBAR (main_window.toolbar_find), main_window.toolbar_find_goto_entry,"Go to line",NULL);
+	gtk_toolbar_append_widget(GTK_TOOLBAR (main_window.toolbar_find), main_window.toolbar_find_goto_entry, _("Go to line"),NULL);
 	gtk_signal_connect_after(GTK_OBJECT(main_window.toolbar_find_goto_entry), "activate", GTK_SIGNAL_FUNC(goto_line_activate), NULL);
 }
 
@@ -292,7 +292,7 @@ static void main_window_fill_panes(void)
 
 	box2 = gtk_hbox_new(FALSE, 0);
 	gtk_widget_show(box2);
-	main_window.treeviewlabel = gtk_label_new("FILE: ");
+	main_window.treeviewlabel = gtk_label_new(_("FILE: "));
 	gtk_label_set_justify(GTK_LABEL(main_window.treeviewlabel), GTK_JUSTIFY_LEFT);
 	gtk_widget_show(main_window.treeviewlabel);
 	gtk_box_pack_start(GTK_BOX(box2), main_window.treeviewlabel, FALSE, FALSE, 0);
@@ -309,7 +309,7 @@ static void main_window_fill_panes(void)
 	main_window.classtreeselect = gtk_tree_view_get_selection (GTK_TREE_VIEW (main_window.classtreeview));
 	gtk_tree_selection_set_mode (main_window.classtreeselect, GTK_SELECTION_SINGLE);
 	renderer = gtk_cell_renderer_text_new ();
-	column = gtk_tree_view_column_new_with_attributes ("Name",
+	column = gtk_tree_view_column_new_with_attributes (_("Name"),
 	         renderer, "text", NAME_COLUMN, NULL);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (main_window.classtreeview), column);
 
@@ -342,7 +342,7 @@ static void main_window_fill_panes(void)
 	main_window.lint_view = gtk_tree_view_new ();
 	gtk_container_add (GTK_CONTAINER (main_window.scrolledwindow1), main_window.lint_view);
 	main_window.lint_renderer = gtk_cell_renderer_text_new ();
-	main_window.lint_column = gtk_tree_view_column_new_with_attributes ("Syntax Check Output",
+	main_window.lint_column = gtk_tree_view_column_new_with_attributes (_("Syntax Check Output"),
 	                          main_window.lint_renderer,
 	                          "text", 0,
 	                          NULL);
@@ -473,23 +473,23 @@ void update_app_title(void)
 			title = get_differing_part_editor(main_window.current_editor);
 			if (title) {
 				if (main_window.current_editor->saved == TRUE) {
-					g_string_append(title," - gPHPEdit");
+					g_string_append(title, _(" - gPHPEdit"));
 				}
 				else {
-					g_string_append(title," (amended) - gPHPEdit");
+					g_string_append(title, _(" (amended) - gPHPEdit"));
 				}
 			}
 			else {
-				title = g_string_new("gPHPEdit");
+				title = g_string_new(_("gPHPEdit"));
 			}
 			gtk_window_set_title(GTK_WINDOW(main_window.window), title->str);
 		}
 		else {
-			gtk_window_set_title(GTK_WINDOW(main_window.window), "gPHPEdit");
+			gtk_window_set_title(GTK_WINDOW(main_window.window), _("gPHPEdit"));
 		}
 	}
 	else {
-		gtk_window_set_title(GTK_WINDOW(main_window.window), "gPHPEdit");
+		gtk_window_set_title(GTK_WINDOW(main_window.window), _("gPHPEdit"));
 	}
 }
 
@@ -558,6 +558,16 @@ int plugin_discover_type(GString *filename)
 	return type;
 }
 
+gint sort_plugin_func(gconstpointer a, gconstpointer b)
+{
+	Plugin *plugina = (Plugin *)a;
+	Plugin *pluginb = (Plugin *)b;
+
+	if (strcmp(plugina->name, pluginb->name) < 0) {
+		return -1;
+	}
+	return 1;
+}
 
 void plugin_discover_available(void)
 {
@@ -577,6 +587,7 @@ void plugin_discover_available(void)
 				// Recommended by __tim in #gtk+ on irc.freenode.net 27/10/2004 11:30
 				plugin = g_new0 (Plugin, 1);
 				plugin->name = g_strdup(plugin_name);
+				// TODO: Could do with replacing ' in name with \' for spawn
 				filename = g_string_new(plugin->name);
 				filename = g_string_prepend(filename, user_plugin_dir->str);
 				plugin->filename = filename;
@@ -608,6 +619,9 @@ void plugin_discover_available(void)
 			g_dir_close(dir);			
 		}
 	}
+
+	Plugins = g_list_sort(Plugins, sort_plugin_func);
+
 	//g_print ("FOUND ALL PLUGINS\n");
 }
 
@@ -627,7 +641,7 @@ void plugin_create_menu_items()
 		//g_print ("Plugin %d:%s\n", num_plugin, plugin->filename);
 		
 		//g_print("Getting child widget\n");
-		
+				
 		bin = GTK_BIN(plugin_menu[num_plugin].widget);
 				
 		//g_print("Bin is %p\n", bin);
@@ -707,8 +721,11 @@ void plugin_exec(gint plugin_num)
 				gtk_scintilla_replace_sel(GTK_SCINTILLA(main_window.current_editor->scintilla), data);
 			}
 		}
+		else if (g_strncasecmp(stdout, "MESSAGE", MIN(strlen(stdout),7))==0) {
+			  info_dialog(plugin->name, data);
+		}
 		else if (g_strncasecmp(stdout, "OPEN", MIN(strlen(stdout), 4))==0) {
-			g_print("OPENING %s\n", data);
+			if (DEBUG_MODE) { g_print("DEBUG: main_window.c:plugin_exec: Opening file :date: %s\n", data); }
 			switch_to_file_or_open(data, 0);
 		}
 		else if (g_strncasecmp(stdout, "DEBUG", MIN(strlen(stdout), 5))==0) {
@@ -744,8 +761,9 @@ void main_window_update_reopen_menu(void)
 	GString *key;
 	//gchar *short_filename;
 	guint entry;
+	GtkBin *bin = NULL;
 	
-	for (entry=0; entry<=9; entry++) {
+	for (entry=0; entry<NUM_REOPEN_MAX; entry++) {
 		key = g_string_new("gPHPEdit/recent/");
 		g_string_append_printf(key, "%d=NOTFOUND", entry);
 		full_filename = gnome_config_get_string (key->str);
@@ -754,8 +772,11 @@ void main_window_update_reopen_menu(void)
 		//g_print("Recent DEBUG: Entry %d: %s\n", entry, full_filename);
 		
 		if (strcmp(full_filename, "NOTFOUND")!=0) {
-			gtk_label_set_text(GTK_LABEL(gtk_bin_get_child(GTK_BIN(recent_menu[entry].widget))), full_filename);
-			gtk_widget_show(recent_menu[entry].widget);
+			bin = GTK_BIN(recent_menu[entry].widget);
+			if (bin) {
+				gtk_label_set_text(GTK_LABEL(gtk_bin_get_child(bin)), full_filename);
+				gtk_widget_show(recent_menu[entry].widget);
+			}
 		}
 		else {
 			gtk_widget_hide(recent_menu[entry].widget);
@@ -772,7 +793,7 @@ void main_window_add_to_reopen_menu(gchar *full_filename)
 	
 	// Find current filename in list
 	found_id = -1;
-	for (entry=0; entry<=9; entry++) {
+	for (entry=0; entry<NUM_REOPEN_MAX; entry++) {
 		key = g_string_new("gPHPEdit/recent/");
 		g_string_append_printf(key, "%d=NOTFOUND", entry);
 		found = gnome_config_get_string (key->str);
@@ -786,7 +807,7 @@ void main_window_add_to_reopen_menu(gchar *full_filename)
 	
 	// if not found, drop the last one off the end (i.e. pretend it was found in the last position)
 	if (found_id == -1) {
-		found_id = 9;
+		found_id = NUM_REOPEN_MAX-1;
 	}
 
 	// replace from found_id to 1 with entry above
@@ -814,9 +835,9 @@ void main_window_create(void)
 {
 	gnome_window_icon_set_default_from_file(PACKAGE_PIXMAPS_DIR "/" GPHPEDIT_PIXMAP_ICON);
 
-	main_window.window = gnome_app_new ("gPHPEdit", "gPHPEdit");
-	preferences_load();
-
+	main_window.window = gnome_app_new (_("gPHPEdit"), _("gPHPEdit"));
+	preferences_apply();
+	
 	gnome_app_create_menus (GNOME_APP (main_window.window), menubar1_uiinfo);
 	main_window_create_appbar();
 	gnome_app_install_menu_hints (GNOME_APP (main_window.window), menubar1_uiinfo);
