@@ -1,6 +1,6 @@
 /* This file is part of gPHPEdit, a GNOME2 PHP Editor.
  
-   Copyright  2003-2004 Andy Jeffries
+   Copyright (C) 2003-2005 Andy Jeffries
       andy@gphpedit.org
 	  
    For more information or to find the latest release, visit our 
@@ -98,7 +98,6 @@ void session_reopen(void)
 	int focus_tab=-1;
 	gboolean focus_this_one = FALSE;
 	GString *target;
-	GnomeVFSURI *uri;
 
 	session_file = g_string_new( g_get_home_dir());
 	session_file = g_string_append(session_file, "/.gphpedit/session");
@@ -135,16 +134,10 @@ void session_reopen(void)
 				}
 			}
 			else {
-				uri = gnome_vfs_uri_new (filename);
-
-				if (gnome_vfs_uri_exists(uri)) {
-					switch_to_file_or_open(filename,0);
-					if (focus_this_one && (main_window.current_editor)) {
-						focus_tab = gtk_notebook_page_num(GTK_NOTEBOOK(main_window.notebook_editor),main_window.current_editor->scintilla);
-					}
+				switch_to_file_or_open(filename,0);
+				if (focus_this_one && (main_window.current_editor)) {
+					focus_tab = gtk_notebook_page_num(GTK_NOTEBOOK(main_window.notebook_editor),main_window.current_editor->scintilla);
 				}
-				
-				gnome_vfs_uri_unref (uri);
 			}
 			
 			focus_this_one=FALSE;
@@ -215,10 +208,13 @@ gboolean can_all_tabs_be_saved(void)
 
 	is_app_closing = TRUE;
 	for(walk = editors; walk!= NULL; walk = g_slist_next(walk)) {
+		if (g_slist_length(editors) == 0) {
+			break;
+		}
 		editor = walk->data;
-		if (editor) {
-			if (!editor->saved && editor->type==TAB_FILE) {
-				saved = try_save_page(editor);
+		if (editor && (editor->scintilla || editor->help_scrolled_window)) {
+			if (!editor->saved && editor->type!=TAB_HELP) {
+				saved = try_save_page(editor, FALSE);
 				if (saved==FALSE) {
 					is_app_closing = FALSE;
 					return FALSE;
@@ -234,7 +230,7 @@ gboolean can_all_tabs_be_saved(void)
 gboolean main_window_delete_event(GtkWidget *widget,
                                   GdkEvent *event, gpointer user_data)
 {
-	gboolean cancel_quit;
+	gboolean cancel_quit = FALSE;
 
 	cancel_quit = !can_all_tabs_be_saved();
 
@@ -261,6 +257,8 @@ gint main_window_key_press_event(GtkWidget   *widget,
 {
 	guint current_pos;
 	guint current_line;
+	guint search_length;
+	gchar *search_buffer;
 	gchar *member_function_buffer;
 	guint member_function_length;
 	guint wordStart;
@@ -280,7 +278,16 @@ gint main_window_key_press_event(GtkWidget   *widget,
 			if ((event->state & GDK_SHIFT_MASK)==GDK_SHIFT_MASK) {
 				return FALSE;
 			}
-			gtk_widget_grab_focus(GTK_WIDGET(main_window.toolbar_find_search_entry));
+			if (main_window.current_editor && main_window.current_editor->type != TAB_HELP) {
+				wordStart = gtk_scintilla_get_selection_start(GTK_SCINTILLA(main_window.current_editor->scintilla));
+				wordEnd = gtk_scintilla_get_selection_end(GTK_SCINTILLA(main_window.current_editor->scintilla));
+				if (wordStart != wordEnd && (wordEnd-wordStart)<=25) {
+					   search_buffer = gtk_scintilla_get_text_range (GTK_SCINTILLA(main_window.current_editor->scintilla), wordStart, wordEnd, &search_length);
+					   gtk_entry_set_text(GTK_ENTRY(main_window.toolbar_find_search_entry), search_buffer);
+				}
+				gtk_widget_grab_focus(GTK_WIDGET(main_window.toolbar_find_search_entry));
+				//gtk_editable_select_region(GTK_EDITABLE(main_window.toolbar_find_search_entry),0, -1);
+			}
 			return TRUE;
 		}
 		else if ((event->state & GDK_CONTROL_MASK)==GDK_CONTROL_MASK && ((event->keyval == GDK_g) || (event->keyval == GDK_G))) {
@@ -352,7 +359,7 @@ void reopen_recent(GtkWidget *widget, gpointer data)
 	GString *key;
 	
 	key = g_string_new("gPHPEdit/recent/");
-	g_string_append_printf(key, "%d=NOTFOUND", (int)data);
+	g_string_append_printf(key, "%d=NOTFOUND", (gulong)data); // Was (gint)
 	filename = gnome_config_get_string (key->str);
 	g_string_free(key, TRUE);
 
@@ -363,7 +370,7 @@ void reopen_recent(GtkWidget *widget, gpointer data)
 
 void run_plugin(GtkWidget *widget, gpointer data)
 {
-	plugin_exec((gint) data);
+	plugin_exec((gulong) data);// was (gint)
 }
 
 gchar *get_gnome_vfs_dirname(gchar *filename)
@@ -410,7 +417,7 @@ void on_openselected1_activate(GtkWidget *widget)
 	gchar *ac_buffer;
 	gint ac_length;
 	GString *file;
-	GnomeVFSURI *uri=NULL;
+	GnomeVFSURI *uri;
 
 	if (main_window.current_editor == NULL || main_window.current_editor->type == TAB_HELP) 
 		return;
@@ -420,37 +427,33 @@ void on_openselected1_activate(GtkWidget *widget)
 	wordEnd = gtk_scintilla_get_selection_end(GTK_SCINTILLA(main_window.current_editor->scintilla));
 	ac_buffer = gtk_scintilla_get_text_range (GTK_SCINTILLA(main_window.current_editor->scintilla), wordStart, wordEnd, &ac_length);
 
-	//g_print("%s\n", ac_buffer);
-
 	for(li = editors; li!= NULL; li = g_slist_next(li)) {
 		editor = li->data;
 		if (editor) {
 			if (editor->opened_from) {
 				file = get_folder(editor->opened_from);
-				g_print("File: %s\n", file->str);
+				if (DEBUG_MODE) { g_print("DEBUG: main_window_callbacks.c:on_selected1_activate (opened_from) :file->str: %s\n", file->str); }
 			}
 			else {
 				file = get_folder(editor->filename);
-				g_print("File: %s\n", file->str);
+				if (DEBUG_MODE) { g_print("DEBUG: main_window_callbacks.c:on_selected1_activate:file->str: %s\n", file->str); }
 			}
 
-			if (file->str) {
+			if (!strstr(ac_buffer, "://") && file->str) {
 				file = g_string_append(file, ac_buffer);
-				
-				uri = gnome_vfs_uri_new (file->str);
-
-				if (gnome_vfs_uri_exists(uri)) {
-					switch_to_file_or_open(file->str,0);
-					g_string_free(file, TRUE);
-					break;
-				}
-				else {
-					g_string_free(file, TRUE);
-				}
-				gnome_vfs_uri_unref (uri);				
 			}
-			else {
-				g_print("Couldn't open selected file: %s\n", ac_buffer);
+			else if (strstr(ac_buffer, "://")) {
+				file = g_string_new(ac_buffer);
+			}
+
+			uri = gnome_vfs_uri_new (file->str);
+			if (gnome_vfs_uri_exists (uri))
+				switch_to_file_or_open(file->str,0);
+			gnome_vfs_uri_unref (uri);
+			
+			
+			if (file) {
+				g_string_free(file, TRUE);
 			}
 		}
 	}
@@ -466,6 +469,8 @@ void on_open1_activate(GtkWidget *widget)
 	file_selection_box = gtk_file_chooser_dialog_new("Please select files for editing", GTK_WINDOW(main_window.window),
 		GTK_FILE_CHOOSER_ACTION_OPEN, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
 	
+	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER(file_selection_box), FALSE);
+	
 	last_opened_folder = gnome_config_get_string("gPHPEdit/general/last_opened_folder=NOTFOUND");
 	if (DEBUG_MODE) { g_print("DEBUG: main_window_callbacks.c:on_open1_activate:last_opened_folder: %s\n", last_opened_folder); }
 	
@@ -556,17 +561,8 @@ void save_file_as_ok(GtkFileChooser *file_selection_box)
 
 void on_save1_activate(GtkWidget *widget)
 {
-	gchar *write_buffer = NULL;
-	gsize text_length;
-	gchar *filename;
-	GtkWidget *cant_save_dialog;
-	//GString *string;
-	GError *error;
-	gchar *converted_text;
-	guint utf8_size;
-	GnomeVFSHandle *write_handle;
-	GnomeVFSFileSize bytes_written;
-	GnomeVFSResult result;
+	gchar *filename = NULL;
+	GnomeVFSAsyncHandle *fg;
 	
 	if (main_window.current_editor) {
 		filename = main_window.current_editor->filename->str;
@@ -576,61 +572,8 @@ void on_save1_activate(GtkWidget *widget)
 			on_save_as1_activate(widget);
 		}
 		else {
-
-			text_length = gtk_scintilla_get_length(GTK_SCINTILLA(main_window.current_editor->scintilla));
-
-			write_buffer = g_malloc0(text_length+1); // Include terminating null
-
-			if (write_buffer == NULL) {
-				g_warning ("%s", "Cannot allocate write buffer");
-				return;
-			}
-
-			result = gnome_vfs_create (&write_handle, filename, GNOME_VFS_OPEN_WRITE, FALSE, 0777);
-
-			if (result != GNOME_VFS_OK) {
-				cant_save_dialog = gtk_message_dialog_new(GTK_WINDOW(main_window.window), GTK_DIALOG_DESTROY_WITH_PARENT,
-					GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, "Unable to save '%s'.\n\nThis may be due to incorrect permissions or being out of space on the drive", main_window.current_editor->short_filename);
-				// HIG 1.0 says don't use titles for alerts!
-				// gtk_window_set_title(GTK_WINDOW(cant_save_dialog), "Unable to save");
-				gtk_dialog_run(GTK_DIALOG (cant_save_dialog));
-				gtk_widget_destroy(cant_save_dialog);
-				return;
-			}
-
-			result = gnome_vfs_seek (write_handle, GNOME_VFS_SEEK_START, 0);
-			
-			gtk_scintilla_get_text(GTK_SCINTILLA(main_window.current_editor->scintilla), text_length+1, write_buffer);
-
-			// If we converted to UTF-8 when loading, convert back to the locale to save
-			if (main_window.current_editor->converted_to_utf8) {
-				converted_text = g_locale_from_utf8(write_buffer, text_length, NULL, &utf8_size, &error);
-				if (error != NULL) {
-					g_print("UTF-8 Error: %s\n", error->message);
-					g_error_free(error);			
-				}
-				else {
-					//g_print("DEBUG: Converted size: %d\n", utf8_size);
-					g_free(write_buffer);
-					write_buffer = converted_text;
-					text_length = utf8_size;
-				}
-			}
-			
-			result = gnome_vfs_write (write_handle, write_buffer, text_length, &bytes_written);
-  			if (result != GNOME_VFS_OK) {
-				g_print("Unable to save, GNOME VFS ERROR: %d\n", result);
-				//return print_error (result, output_uri_string);
-			}
-
-			/* Close the file pointer as soon as it's finished with */
-			gnome_vfs_close(write_handle);
-
-			g_free (write_buffer);
-
-			gtk_scintilla_set_save_point (GTK_SCINTILLA(main_window.current_editor->scintilla));
+			gnome_vfs_async_create(&fg, filename, GNOME_VFS_OPEN_WRITE, FALSE, 0755, GNOME_VFS_PRIORITY_DEFAULT, tab_file_save_opened, main_window.current_editor);
 		}
-		classbrowser_update();
 	}
 }
 
@@ -688,10 +631,12 @@ void on_save_as1_activate(GtkWidget *widget)
 
 	if (main_window.current_editor) {
 		// Create the selector widget
-		file_selection_box = gtk_file_chooser_dialog_new ("Please type the filename to save as...", 
+		file_selection_box = gtk_file_chooser_dialog_new (_("Please type the filename to save as..."), 
 			GTK_WINDOW(main_window.window), GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 			GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
 
+		gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER(file_selection_box), FALSE);
+	
 		last_opened_folder = gnome_config_get_string("gPHPEdit/general/last_opened_folder=NOTFOUND");
 		if (DEBUG_MODE) { g_print("DEBUG: main_window_callbacks.c:on_save_as1_activate:last_opened_folder: %s\n", last_opened_folder); }
 	
@@ -731,7 +676,7 @@ void on_reload1_activate(GtkWidget *widget)
 		return;
 
 	if (main_window.current_editor && (main_window.current_editor->saved == FALSE)) {
-		file_revert_dialog = gnome_question_dialog_modal("Are you sure you wish to reload the current file, losing your changes?",
+		file_revert_dialog = gnome_question_dialog_modal(_("Are you sure you wish to reload the current file, losing your changes?"),
 		                     on_reload_confirm,NULL);
 		gnome_dialog_run_and_close(GNOME_DIALOG(file_revert_dialog));
 	}
@@ -772,10 +717,10 @@ void rename_file_ok(GtkFileChooser *file_selection)
 	struct stat st;
 
 	// Extract filename from the file chooser dialog
-	filename = g_string_new(gtk_file_chooser_get_filename(file_selection));
+	filename = g_string_new(gtk_file_chooser_get_uri(file_selection));
 
 	if (stat (filename->str, &st) == 0) {
-		file_exists_dialog = gnome_question_dialog_modal("This file already exists, are you sure you want to overwrite it?",
+		file_exists_dialog = gnome_question_dialog_modal(_("This file already exists, are you sure you want to overwrite it?"),
 		                     save_file_as_confirm_overwrite,filename);
 		gnome_dialog_run_and_close(GNOME_DIALOG(file_exists_dialog));
 	}
@@ -790,7 +735,7 @@ void on_rename1_activate(GtkWidget *widget)
 
 	if (main_window.current_editor) {
 		// Create the selector widget
-		file_selection_box = gtk_file_chooser_dialog_new("Please type the filename to rename this file to...",
+		file_selection_box = gtk_file_chooser_dialog_new(_("Please type the filename to rename this file to..."),
 			GTK_WINDOW(main_window.window), GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 			GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
 		
@@ -849,26 +794,28 @@ void close_page(Editor *editor)
 }
 
 
-gboolean try_save_page(Editor *editor)
+gboolean try_save_page(Editor *editor, gboolean close_if_can)
 {
 	GtkWidget *confirm_dialog;
 	int ret;
 	GString *string;
 
 	string = g_string_new("");
-	g_string_printf(string,"The file '%s' has not been saved since your last changes, are you sure you want to close it and lose these changes?", editor->short_filename);
+	g_string_printf(string, _("The file '%s' has not been saved since your last changes, are you sure you want to close it and lose these changes?"), editor->short_filename);
 	confirm_dialog = gnome_message_box_new (string->str,GNOME_MESSAGE_BOX_WARNING,
-		"Close and _lose changes", "_Save file", "_Cancel closing", NULL);
-	g_string_printf(string,"Unsaved changes to '%s'", editor->filename->str);
+		_("Close and _lose changes"), _("_Save file"), _("_Cancel closing"), NULL);
+	g_string_printf(string,_("Unsaved changes to '%s'"), editor->filename->str);
 	gtk_window_set_title(GTK_WINDOW(confirm_dialog), string->str);
 	ret = gnome_dialog_run (GNOME_DIALOG (confirm_dialog));
 	switch (ret) {
 		case 0:
-			close_page(editor);
-			editors = g_slist_remove(editors, editor);
-			if (g_slist_length(editors) == 0) {
-				editors = NULL;
-				main_window.current_editor = NULL;
+			if (close_if_can) {
+				close_page(editor);
+				editors = g_slist_remove(editors, editor);
+				if (g_slist_length(editors) == 0) {
+					editors = NULL;
+					main_window.current_editor = NULL;
+				}
 			}
 			return TRUE;
 		case 1:
@@ -882,7 +829,7 @@ gboolean try_save_page(Editor *editor)
 gboolean try_close_page(Editor *editor)
 {
 	if (!editor->saved && editor->type==TAB_FILE) {
-		return try_save_page(editor);
+		return try_save_page(editor, TRUE);
 	}
 	else {
 		close_page(editor);
@@ -905,7 +852,7 @@ void on_close1_activate(GtkWidget *widget)
 		classbrowser_update();
 		update_app_title();
 		if (!gtk_tree_selection_get_selected (main_window.classtreeselect, NULL, &iter)) {
-			gtk_label_set_text(GTK_LABEL(main_window.treeviewlabel), "FILE:");
+			gtk_label_set_text(GTK_LABEL(main_window.treeviewlabel), _("FILE:"));
 		}
 	}
 }
@@ -1043,7 +990,7 @@ void on_selectall1_activate(GtkWidget *widget)
 
 void on_find1_activate(GtkWidget *widget)
 {
-	if (main_window.current_editor) {
+	if (main_window.current_editor && main_window.current_editor->type != TAB_HELP) {
 		if (find_dialog.window1==NULL) {
 			find_create();
 		}
@@ -1197,7 +1144,7 @@ void context_help(GtkWidget *widget)
 	guint wordStart;
 	guint wordEnd;
 
-	if (main_window.current_editor) {
+	if (main_window.current_editor && main_window.current_editor->type != TAB_HELP) {
 		wordStart = gtk_scintilla_get_selection_start(GTK_SCINTILLA(main_window.current_editor->scintilla));
 		wordEnd = gtk_scintilla_get_selection_end(GTK_SCINTILLA(main_window.current_editor->scintilla));
 		if (wordStart != wordEnd) {
@@ -1217,7 +1164,9 @@ void context_help(GtkWidget *widget)
 void on_about1_activate(GtkWidget *widget)
 {
 	const gchar *authors[] = {
-	                             "Andy Jeffries <andy@gphpedit.org>\nTim Jackson <tim@timj.co.uk>\nSven Herzberg <herzi@gnome-de.org>",
+	                             "Andy Jeffries <andy@gphpedit.org>",
+				     "Tim Jackson <tim@timj.co.uk>",
+				     "Sven Herzberg <herzi@gnome-de.org>",
 	                             NULL
 	                         };
 	const gchar *documenters[] = {
@@ -1234,9 +1183,9 @@ void on_about1_activate(GtkWidget *widget)
 		g_error_free (error);
 	}
 	about = gnome_about_new ("gPHPEdit", VERSION,
-	                         "Copyright  2003 Andy Jeffries.",
-                         	"gPHPEdit is a GNOME2 editor specialised for editing PHP "
-	                         "scripts and related files (HTML/CSS/JS).",
+	                         _("Copyright  2003 Andy Jeffries."),
+                         	 _("gPHPEdit is a GNOME2 editor specialised for editing PHP "
+	                         "scripts and related files (HTML/CSS/JS)."),
 	                         (const gchar **) authors, (const gchar **) documenters, NULL,
 	                         pixbuf);
 	gtk_widget_show(about);
@@ -1564,3 +1513,10 @@ void force_perl(GtkWidget *widget)
 		set_editor_to_perl(main_window.current_editor);
 	}
 }
+
+void force_python(GtkWidget *widget)
+{
+	if (main_window.current_editor) {
+		set_editor_to_python(main_window.current_editor);
+	}
+}
