@@ -1,6 +1,6 @@
 /* This file is part of gPHPEdit, a GNOME2 PHP Editor.
  
-   Copyright  2003-2004 Andy Jeffries
+   Copyright (C) 2003-2005 Andy Jeffries
       andy@gphpedit.org
 	  
    For more information or to find the latest release, visit our 
@@ -43,7 +43,7 @@ void check_for_pango_fonts(void)
                                   GTK_DIALOG_MODAL,
 				  GTK_MESSAGE_ERROR,
                                   GTK_BUTTONS_CLOSE,
-                                  "Your old settings used non-Pango (modern) fonts.  gPHPEdit no longer supports old fonts, your preferences have been updated.");
+                                  _("Your old settings used non-Pango (modern) fonts.  gPHPEdit no longer supports old fonts, your preferences have been updated."));
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (dialog);
 
@@ -105,13 +105,12 @@ void check_for_pango_fonts(void)
 }
 
 
-void preferences_load(void)
+void preferences_apply(void)
 {
 	gint root_x;
 	gint root_y;
 	gint width;
 	gint height;
-	gboolean ignore;
 
 	root_x = gnome_config_get_int ("gPHPEdit/main_window/x=20");
 	root_y = gnome_config_get_int ("gPHPEdit/main_window/y=20");
@@ -120,7 +119,11 @@ void preferences_load(void)
 
 	gtk_window_move(GTK_WINDOW(main_window.window), root_x, root_y);
 	gtk_window_set_default_size(GTK_WINDOW(main_window.window), width, height);
-	
+}
+
+void preferences_load(void)
+{
+	gboolean ignore;
 
 	preferences.set_sel_back = gnome_config_get_int ("gPHPEdit/default_style/selection=11250603");
 	preferences.marker_back = gnome_config_get_int ("gPHPEdit/default_style/bookmark=15908608");
@@ -139,6 +142,7 @@ void preferences_load(void)
 	//preferences.auto_indent_after_brace = gnome_config_get_int ("gPHPEdit/defaults/autoindentafterbrace=1");
 	preferences.save_session = gnome_config_get_bool_with_default("gPHPEdit/defaults/save_session=true", &ignore);
 	preferences.use_tabs_instead_spaces = gnome_config_get_bool_with_default ("gPHPEdit/defaults/use_tabs_instead_spaces=true", &ignore);
+	preferences.single_instance_only = gnome_config_get_bool_with_default ("gPHPEdit/defaults/single_instance_only=false", &ignore);
 	
 	preferences.php_file_extensions = gnome_config_get_string("gPHPEdit/defaults/php_file_extensions=php,inc,phtml,php3,xml,htm,html");
 	
@@ -536,7 +540,7 @@ void preferences_save(void)
 	gnome_config_set_int ("gPHPEdit/line_numbers/size", preferences.line_number_size);
 	gnome_config_set_bool ("gPHPEdit/line_numbers/italic", preferences.line_number_italic);
 	gnome_config_set_bool ("gPHPEdit/line_numbers/bold", preferences.line_number_bold);
-
+	
 	gnome_config_set_int ("gPHPEdit/default_style/selection", preferences.set_sel_back);
 	gnome_config_set_int ("gPHPEdit/default_style/bookmark", preferences.marker_back);
 	gnome_config_set_string ("gPHPEdit/locations/php_binary", preferences.php_binary_location);
@@ -555,6 +559,7 @@ void preferences_save(void)
 	gnome_config_set_string("gPHPEdit/defaults/php_file_extensions",preferences.php_file_extensions);
 	gnome_config_set_bool("gPHPEdit/defaults/save_session", preferences.save_session);
 	gnome_config_set_bool("gPHPEdit/defaults/use_tabs_instead_spaces", preferences.use_tabs_instead_spaces);
+	gnome_config_set_bool ("gPHPEdit/defaults/single_instance_only", preferences.single_instance_only);
 
 	gnome_config_set_int ("gPHPEdit/html_tag/back", preferences.html_tag_back);
 	gnome_config_set_int ("gPHPEdit/html_tag/fore", preferences.html_tag_fore);
