@@ -1,6 +1,6 @@
 /* This file is part of gPHPEdit, a GNOME2 PHP Editor.
  
-   Copyright  2003-2004 Andy Jeffries
+   Copyright (C) 2003-2005 Andy Jeffries
       andy@gphpedit.org
 	  
    For more information or to find the latest release, visit our 
@@ -26,7 +26,6 @@
 
 #define PLAT_GTK 1
 #include <gtkscintilla.h>
-#include <libgnomevfs/gnome-vfs.h>
 
 #include "tab.h"
 #include "tab_php.h"
@@ -34,6 +33,7 @@
 #include "tab_sql.h"
 #include "tab_cxx.h"
 #include "tab_perl.h"
+#include "tab_python.h"
 #include "calltip.h"
 #include "main_window.h"
 #include "main_window_callbacks.h"
@@ -69,6 +69,7 @@ void debug_dump_editors(void)
 		
 		g_print("Editor number    :%d\n", editor_number);
 		g_print("Scintilla widget :%p\n", editor->scintilla);
+		g_print("Scintilla lexer :%p\n", gtk_scintilla_get_lexer(GTK_SCINTILLA(editor->scintilla)));
 		g_print("Scintilla ID     :%d\n", editor->scintilla_id);
 		g_print("File mtime       :%d\n", editor->file_mtime);
 		g_print("File shortname   :%s\n", editor->short_filename);
@@ -87,6 +88,7 @@ void tab_set_general_scintilla_properties(Editor *editor)
 	gtk_scintilla_set_backspace_unindents(GTK_SCINTILLA(editor->scintilla), 1);
 
 	gtk_scintilla_autoc_set_choose_single(GTK_SCINTILLA (editor->scintilla), FALSE);
+	gtk_scintilla_autoc_set_ignore_case(GTK_SCINTILLA (editor->scintilla), TRUE);
 	gtk_scintilla_autoc_set_drop_rest_of_word(GTK_SCINTILLA (editor->scintilla), FALSE);
 
 	gtk_scintilla_set_code_page(GTK_SCINTILLA(editor->scintilla), 65001); // Unicode code page
@@ -193,93 +195,203 @@ static void tab_set_event_handlers(Editor *editor)
 	gtk_signal_connect (GTK_OBJECT (editor->scintilla), "update_ui", GTK_SIGNAL_FUNC (update_ui), NULL);
 }
 
-int gnome_vfs_print_error (GnomeVFSResult result, const char *uri_string)
+void report_vfs_error(gchar *name, gchar *desc, GnomeVFSResult result, GtkWindow *win)
 {
-  const char *error_string;
-  GString *str;
-  // get the string corresponding to this GnomeVFSResult value 
-  error_string = gnome_vfs_result_to_string (result);
-  str = g_string_new(NULL);
-  g_string_sprintf(str, "Error %s occured opening location %s\n", error_string, uri_string);
-  info_dialog("GNOME-VFS", str->str);
+	GtkDialog *dialog;
+	gchar *err_title, *err_message;
+	
+	if (name) {
+		err_title = g_strdup_printf("File access error:\n %s: ", name);
+	}
+	else {
+		err_title = g_strdup_printf("File access error: \n");
+	}
+	
+	if (result < GNOME_VFS_NUM_ERRORS) {
+		err_message = g_strdup(gnome_vfs_result_to_string(result));
+	}
+	else {
+		err_message = g_strdup_printf("%s\n", desc ? desc : "");
+	}
+	dialog = GTK_DIALOG(gtk_message_dialog_new(win, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "%s %s", err_title, err_message));
+	g_signal_connect_swapped(dialog, "response", G_CALLBACK(gtk_widget_destroy), dialog);
+	g_object_set(dialog, "title", "", NULL);
+	gtk_widget_show_all(GTK_WIDGET(dialog));
+	g_free(err_title);
+	g_free(err_message);
+}
+
+int gnome_vfs_print_error (GnomeVFSResult result, char *uri_string)
+{
+  report_vfs_error(uri_string, NULL, result, GTK_WINDOW(main_window.window));
   return 1;
 }
 
-void tab_load_file(Editor *editor)
+void tab_file_write(GnomeVFSAsyncHandle *fd, GnomeVFSResult result, gconstpointer buffer, GnomeVFSFileSize bytes_requested, GnomeVFSFileSize bytes_received, gpointer li_ptr)
 {
-	GnomeVFSFileSize size;
-	gchar *buffer;
-	gchar *converted_text;
-	guint utf8_size;
+  	Editor *editor = (Editor *)li_ptr;
+	
+	if (result != GNOME_VFS_OK) {
+		gnome_vfs_print_error(result, editor->filename->str);
+		return;
+	}
+
+	gtk_scintilla_set_save_point (GTK_SCINTILLA(editor->scintilla));
+	register_file_opened(editor->filename->str);
+	classbrowser_update();
+}
+
+void tab_file_save_opened(GnomeVFSAsyncHandle *fd, GnomeVFSResult result, gpointer li_ptr)
+{
+	gchar *write_buffer = NULL;
+	gsize text_length;
 	GError *error = NULL;
-    GnomeVFSHandle *read_handle;
-    GnomeVFSFileSize bytes_read;
-    GnomeVFSResult result;
-	GnomeVFSFileInfo info;
+	gchar *converted_text = NULL;
+	gsize utf8_size; // was guint
+	GnomeVFSFileSize bytes_written;
+	Editor *editor = (Editor *)li_ptr;
 	
-	result=gnome_vfs_get_file_info(editor->filename->str, &info,0);
 	if (result != GNOME_VFS_OK) {
 		gnome_vfs_print_error(result, editor->filename->str);
 		return;
 	}
-	size = info.size;
-	gtk_scintilla_clear_all(GTK_SCINTILLA (editor->scintilla));
 
-	buffer = (gchar *)g_malloc (size);
-	if (buffer == NULL && size != 0) {
-		/* This is funny in unix, but never hurts */
-		g_warning ("This file is too big. Unable to allocate memory.");
-		g_free (buffer);
+	text_length = gtk_scintilla_get_length(GTK_SCINTILLA(main_window.current_editor->scintilla));
+
+	write_buffer = g_malloc0(text_length+1); // Include terminating null
+
+	if (write_buffer == NULL) {
+		g_warning ("%s", "Cannot allocate write buffer");
 		return;
 	}
-	
-	result = gnome_vfs_open (&read_handle, editor->filename->str, GNOME_VFS_OPEN_READ);
+
+	gtk_scintilla_get_text(GTK_SCINTILLA(main_window.current_editor->scintilla), text_length+1, write_buffer);
+
+	// If we converted to UTF-8 when loading, convert back to the locale to save
+	if (main_window.current_editor->converted_to_utf8) {
+		converted_text = g_locale_from_utf8(write_buffer, text_length, NULL, &utf8_size, &error);
+		if (error != NULL) {
+			g_print("UTF-8 Error: %s\n", error->message);
+			g_error_free(error);			
+		}
+		else {
+			//g_print("DEBUG: Converted size: %d\n", utf8_size);
+			g_free(write_buffer);
+			write_buffer = converted_text;
+			text_length = utf8_size;
+		}
+	}
+		
+	gnome_vfs_async_write (fd, write_buffer, text_length, tab_file_write, li_ptr);
+	//g_free (write_buffer);
+}
+
+void tab_file_closed(GnomeVFSAsyncHandle *fd, GnomeVFSResult result, gpointer li_ptr)
+{
+	Editor *editor = (Editor *)li_ptr;
+
 	if (result != GNOME_VFS_OK) {
 		gnome_vfs_print_error(result, editor->filename->str);
-		g_free (buffer);
-		return;
 	}
+}
 
-	result = gnome_vfs_read (read_handle, buffer, size, &bytes_read);
-  	if (result != GNOME_VFS_OK) {
-		gnome_vfs_print_error(result, editor->filename->str);
-		g_free (buffer);
-		return;
-	}
+void tab_validate_buffer_and_insert(gpointer buffer, Editor *editor)
+{
+	gchar *converted_text;
+	gsize utf8_size;// was guint
+	GError *error = NULL;
 	
-	/* Close the file pointer as soon as it's finished with */
-	gnome_vfs_close(read_handle);
-
-	if (size != bytes_read)
-		info_dialog("GNOME-VFS", "Didn't read full size of file");
-
-	if (g_utf8_validate(buffer, bytes_read, NULL)) {
-		gtk_scintilla_add_text(GTK_SCINTILLA (editor->scintilla), bytes_read, buffer);
+	if (g_utf8_validate(buffer, editor->file_size, NULL)) {
+		//g_print("Valid UTF8 according to gnome\n");
+		gtk_scintilla_add_text(GTK_SCINTILLA (editor->scintilla), editor->file_size, buffer);
 		editor->converted_to_utf8 = FALSE;
 	}
 	else {
 		// Used for testing as my locale isn't set
 		// converted_text = g_convert(buffer, nchars, "UTF-8", "ISO-8859-15", NULL, &utf8_size, &error);
-		converted_text = g_locale_to_utf8(buffer, bytes_read, NULL, &utf8_size, &error);
+		converted_text = g_locale_to_utf8(buffer, editor->file_size, NULL, &utf8_size, &error);
 		if (error != NULL) {
-			g_print("UTF-8 Error: %s\n", error->message);
+			g_print("gPHPEdit UTF-8 Error: %s\n", error->message);
 			g_error_free(error);
-			gtk_scintilla_add_text(GTK_SCINTILLA (editor->scintilla), bytes_read, buffer);
+			gtk_scintilla_add_text(GTK_SCINTILLA (editor->scintilla), editor->file_size, buffer);
 		}
 		else {
-			//g_print("Converted size: %d\n", utf8_size);
+			g_print("Converted to UTF-8 size: %d\n", utf8_size);
 			gtk_scintilla_add_text(GTK_SCINTILLA (editor->scintilla), utf8_size, converted_text);
 			g_free(converted_text);
 			editor->converted_to_utf8 = TRUE;
 		}
 	}
+}
 
+void tab_reset_scintilla_after_open(Editor *editor)
+{
 	gtk_scintilla_empty_undo_buffer(GTK_SCINTILLA(editor->scintilla));
 	gtk_scintilla_set_save_point(GTK_SCINTILLA(editor->scintilla));
-	gtk_scintilla_goto_pos(GTK_SCINTILLA(editor->scintilla), 0);
+	gtk_scintilla_goto_line(GTK_SCINTILLA(editor->scintilla), editor->current_line);
+	gtk_scintilla_scroll_caret(GTK_SCINTILLA(editor->scintilla));
+	
 	gtk_scintilla_grab_focus(GTK_SCINTILLA(editor->scintilla));
+}
 
-	g_free (buffer);
+void tab_file_read(GnomeVFSAsyncHandle *fd, GnomeVFSResult result, gpointer buffer, GnomeVFSFileSize bytes_requested, GnomeVFSFileSize bytes_received, gpointer li_ptr)
+{
+	Editor *editor = (Editor *)li_ptr;
+
+	if ((result != GNOME_VFS_OK) && (result != GNOME_VFS_ERROR_EOF)) {
+		gnome_vfs_print_error(result, editor->filename->str);
+		return;
+	}
+
+	// Clear scintilla buffer
+	gtk_scintilla_clear_all(GTK_SCINTILLA (editor->scintilla));
+	
+	//g_print("BUFFER=\n%s\n-------------------------------------------\n", buffer);
+	
+	tab_validate_buffer_and_insert(buffer, editor);
+	tab_reset_scintilla_after_open(editor);
+	
+	g_free(buffer);
+	gnome_vfs_async_close(fd, tab_file_closed, li_ptr);
+}
+
+void tab_file_opened(GnomeVFSAsyncHandle *fd, GnomeVFSResult result, gpointer li_ptr)
+{
+	gpointer buffer;
+	Editor *editor = (Editor *)li_ptr;
+	
+	if (result != GNOME_VFS_OK) {
+		gnome_vfs_print_error(result, editor->filename->str);
+		return;
+	}
+	
+	buffer = g_malloc(editor->file_size);
+	gnome_vfs_async_read(fd, buffer, editor->file_size, tab_file_read, li_ptr);
+}
+
+ void tab_load_file(Editor *editor)
+{
+	GnomeVFSFileInfo *file_stats;
+	GnomeVFSResult result;
+	GnomeVFSAsyncHandle *fd;
+	
+	// Store current position for returning to
+	editor->current_pos = gtk_scintilla_get_current_pos(GTK_SCINTILLA(editor->scintilla));
+	editor->current_line = gtk_scintilla_line_from_position(GTK_SCINTILLA(editor->scintilla), editor->current_pos);
+
+	// Try getting file size
+	file_stats = gnome_vfs_file_info_new();
+	result = gnome_vfs_get_file_info(editor->filename->str, file_stats, GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
+	if (result != GNOME_VFS_OK) {
+		// Print error and return if no
+		gnome_vfs_print_error(result, editor->filename->str);
+		return;
+	}
+	
+	editor->file_size = file_stats->size;
+	
+	// Open file
+	gnome_vfs_async_open(&fd, editor->filename->str, GNOME_VFS_OPEN_READ, GNOME_VFS_PRIORITY_DEFAULT, tab_file_opened, editor);
 }
 
 
@@ -408,7 +520,7 @@ GString *tab_help_find_helpfile(gchar *command)
 {
 	GString *long_filename = NULL;
 	
-	// For Redhat/Fedora Core and sensible distrubutions...
+	// For Redhat/Fedora Core and other sensible distrubutions...
 	long_filename = tab_help_try_filename("/usr/share/doc/phpmanual/function.", command, ".html");
 	if (long_filename)
 		return long_filename;
@@ -419,6 +531,17 @@ GString *tab_help_find_helpfile(gchar *command)
 	if (long_filename)
 		return long_filename;
 
+	// For Debian, Ubuntu and sensible distrubutions...
+	long_filename = tab_help_try_filename("/usr/share/doc/phpdoc/html/function.", command, ".html");
+	if (long_filename)
+		return long_filename;
+	long_filename = tab_help_try_filename("/usr/share/doc/phpdoc/html/ref.", command, ".html");
+	if (long_filename)
+		return long_filename;
+	long_filename = tab_help_try_filename("/usr/share/doc/phpdoc/html/", command, NULL);
+	if (long_filename)
+		return long_filename;
+
 	// For Gentoo, as much as I love it - it's twatty to put docs in a version specific folder like this!
 	long_filename = tab_help_try_filename("/usr/doc/php-docs-200403/html/function.", command, ".html");
 	if (long_filename)
@@ -430,13 +553,13 @@ GString *tab_help_find_helpfile(gchar *command)
 	if (long_filename)
 		return long_filename;
 
-	long_filename = tab_help_try_filename("/usr/share/doc/php-docs-200403/html/function.", command, ".html");
+	long_filename = tab_help_try_filename("/usr/share/doc/php-docs-20050822/html/function.", command, ".html");
 	if (long_filename)
 		return long_filename;
-	long_filename = tab_help_try_filename("/usr/share/doc/php-docs-200403/html/ref.", command, ".html");
+	long_filename = tab_help_try_filename("/usr/share/doc/php-docs-20050822/html/ref.", command, ".html");
 	if (long_filename)
 		return long_filename;
-	long_filename = tab_help_try_filename("/usr/share/doc/php-docs-200403/html/", command, NULL);
+	long_filename = tab_help_try_filename("/usr/share/doc/php-docs-20050822/html/", command, NULL);
 	if (long_filename)
 		return long_filename;
 
@@ -493,7 +616,7 @@ gboolean tab_create_help(Editor *editor, GString *filename)
 	long_filename = tab_help_find_helpfile(filename->str);
 	if (!long_filename) {
 		dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
-			"Could not find the required command in the online help");
+			_("Could not find the required command in the online help"));
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (dialog);
 		return FALSE;
@@ -630,6 +753,20 @@ gboolean is_perl_file(gchar *filename)
 	return FALSE;
 }
 
+gboolean is_python_file(gchar *filename)
+{
+	gchar *file_extension;
+
+	file_extension = strrchr(filename, '.');
+	if (file_extension) {
+		file_extension++;
+		if (strcmp(file_extension, "py") == 0) {
+			return TRUE;
+		}
+	}
+	return FALSE;
+}
+
 gboolean is_cxx_file(gchar *filename)
 {
 	gchar *file_extension;
@@ -691,6 +828,12 @@ void set_editor_to_perl(Editor *editor)
 	editor->type = TAB_PERL;
 }
 
+void set_editor_to_python(Editor *editor)
+{
+	tab_python_set_lexer(editor);
+	editor->type = TAB_PYTHON;
+}
+
 void tab_check_php_file(Editor *editor)
 {
 	if (is_php_file(editor->filename->str)) {
@@ -713,6 +856,13 @@ void tab_check_perl_file(Editor *editor)
 	}
 }
 
+void tab_check_python_file(Editor *editor)
+{
+	if (is_python_file(editor->filename->str)) {
+		set_editor_to_python(editor);
+	}
+}
+
 void tab_check_cxx_file(Editor *editor)
 {
 	if (is_cxx_file(editor->filename->str)) {
@@ -804,7 +954,9 @@ gboolean tab_create_new(gint type, GString *filename)
 	gboolean result;
 	gboolean file_created = FALSE;
 	GnomeVFSURI *uri = NULL;
+	gchar * buffer = NULL;
 
+	if (DEBUG_MODE) { g_print("DEBUG: tab.c:tab_create_new:filename->str: %s\n", filename->str); }
 	if (filename != NULL) {
 		if (strstr(filename->str, ":")==NULL) {
 			cwd = g_get_current_dir();
@@ -818,8 +970,8 @@ gboolean tab_create_new(gint type, GString *filename)
 		uri = gnome_vfs_uri_new(abs_path);
 		if (!gnome_vfs_uri_exists(uri) && type!=TAB_HELP) {
 			dialog_message = g_string_new("");
-			g_string_sprintf(dialog_message, "The file %s was not found.\n\nWould you like to create it as an empty document?", filename->str);
-			result = yes_no_dialog("File not found", dialog_message->str);
+			g_string_sprintf(dialog_message, _("The file %s was not found.\n\nWould you like to create it as an empty document?"), filename->str);
+			result = yes_no_dialog(_("File not found"), dialog_message->str);
 			g_string_free(dialog_message, TRUE);
 			gnome_vfs_uri_unref(uri);
 			if (result != 0) {
@@ -847,7 +999,7 @@ gboolean tab_create_new(gint type, GString *filename)
 		editor->scintilla = gtk_scintilla_new();
 		tab_set_general_scintilla_properties(editor);
 
-		editor->label = gtk_label_new ("Untitled");
+		editor->label = gtk_label_new (_("Untitled"));
 		gtk_widget_show (editor->label);
 
 		if (abs_path != NULL) {
@@ -861,12 +1013,13 @@ gboolean tab_create_new(gint type, GString *filename)
 			tab_check_css_file(editor);
 			tab_check_cxx_file(editor);
 			tab_check_perl_file(editor);
+			tab_check_python_file(editor);
 			tab_check_sql_file(editor);
 			classbrowser_update();
 			editor->is_untitled=FALSE;
 		}
 		else {
-			editor->filename = g_string_new("Untitled");
+			editor->filename = g_string_new(_("Untitled"));
 			editor->short_filename = g_strdup(editor->filename->str);
 			if (main_window.current_editor) {
 				editor->opened_from = get_folder(main_window.current_editor->filename);
@@ -878,7 +1031,7 @@ gboolean tab_create_new(gint type, GString *filename)
 		if (!GTK_WIDGET_VISIBLE (editor->scintilla))
 			gtk_widget_show (editor->scintilla);
 		gtk_notebook_append_page (GTK_NOTEBOOK (main_window.notebook_editor), editor->scintilla, editor->label);
-		gtk_scintilla_set_save_point(GTK_SCINTILLA(editor->scintilla));
+ 		gtk_scintilla_set_save_point(GTK_SCINTILLA(editor->scintilla));
 		tab_set_event_handlers(editor);
 		
 		/* Possible problem on the next line, one user reports: 
@@ -1313,7 +1466,7 @@ void update_ui(GtkWidget *scintilla)
 	
 	
 	// If no brace has been found or we aren't editing PHP code
-	if ((current_brace_pos==-1) || (gtk_scintilla_get_line_state(GTK_SCINTILLA(scintilla), current_line)!=274)) {
+	if ((current_brace_pos==-1)) {
 		gtk_scintilla_brace_bad_light(GTK_SCINTILLA(scintilla), -1);// Remove any existing highlight
 		return;
 	}
@@ -1353,6 +1506,7 @@ static void char_added(GtkWidget *scintilla, guint ch)
 	gchar *previous_char_buffer;
 	gint previous_char_buffer_length;
 	guint style;
+	gint type;
 
 	current_pos = gtk_scintilla_get_current_pos(GTK_SCINTILLA(scintilla));
 	current_line = gtk_scintilla_line_from_position(GTK_SCINTILLA(scintilla), current_pos);
@@ -1360,32 +1514,23 @@ static void char_added(GtkWidget *scintilla, guint ch)
 	wordEnd = gtk_scintilla_word_end_position(GTK_SCINTILLA(scintilla), current_pos-1, TRUE);
 	current_word_length = wordEnd - wordStart;
 	style = gtk_scintilla_get_style_at(GTK_SCINTILLA(scintilla), current_pos);
+	type = main_window.current_editor->type;
 
 	if (gtk_scintilla_autoc_active(GTK_SCINTILLA(scintilla))==1) {
 		style = 0; // Hack to get around the drop-down not showing in comments, but if it's been forced...	
 	}
 
-	if (main_window.current_editor->type == TAB_PHP && (style != SCE_HPHP_SIMPLESTRING) &&
-		(style != SCE_HPHP_HSTRING) && (style != SCE_HPHP_COMMENTLINE) && (style !=SCE_HPHP_COMMENT)) {
+	if ( ( type != TAB_HELP && 
+		(style != SCE_HPHP_SIMPLESTRING) && (style != SCE_HPHP_HSTRING) && 
+		(style != SCE_HPHP_COMMENTLINE) && (style !=SCE_HPHP_COMMENT)) ) {
 
-		member_function_buffer = gtk_scintilla_get_text_range (GTK_SCINTILLA(scintilla), wordStart-2, wordStart, &member_function_length);
-
-		if (gtk_scintilla_call_tip_active(GTK_SCINTILLA(scintilla)) && ch==')') {
-			gtk_scintilla_call_tip_cancel(GTK_SCINTILLA(scintilla));
-		}
-		else if ((ch == '(') && (gtk_scintilla_get_line_state(GTK_SCINTILLA(main_window.current_editor->scintilla), current_line))==274) {
-			if (calltip_timer_set==FALSE) {
-				calltip_timer_id = g_timeout_add(preferences.calltip_delay, calltip_callback, (gpointer) current_pos);
-				calltip_timer_set=TRUE;
-			}
-		}
-		else if (ch=='\r' || ch=='\n') {
+		if (ch=='\r' || ch=='\n') {
 			//g_print("current_line: %d\n", current_line);
 			if (current_line>0) {
 				previous_line = current_line-1;
 				previous_line_indentation = gtk_scintilla_get_line_indentation(GTK_SCINTILLA(scintilla), previous_line);
 
-				if (1==1) {//preferences.auto_indent_after_brace) {
+				if (1==1) {//TODO: preferences.auto_indent_after_brace) {
 					previous_line_end = gtk_scintilla_get_line_end_position(GTK_SCINTILLA(scintilla), previous_line);
 					previous_char_buffer = gtk_scintilla_get_text_range (GTK_SCINTILLA(scintilla), previous_line_end-1, previous_line_end, &previous_char_buffer_length);
 					if (*previous_char_buffer=='{') {
@@ -1395,103 +1540,68 @@ static void char_added(GtkWidget *scintilla, guint ch)
 
 				indent_line(scintilla, current_line, previous_line_indentation);
 
-				//g_print("previous_line: %d\nprevious_indent: %d\n", previous_line, previous_line_indentation);
+				if (DEBUG_MODE) { g_print("DEBUG: tab.c:char_added:previous_line=%d, previous_indent=%d\n", previous_line, previous_line_indentation); }
 				//gtk_scintilla_goto_pos(GTK_SCINTILLA(scintilla), gtk_scintilla_get_line_end_position(GTK_SCINTILLA(scintilla), current_line));
-				gtk_scintilla_goto_pos(GTK_SCINTILLA(scintilla), gtk_scintilla_position_from_line(GTK_SCINTILLA(scintilla), current_line)+(previous_line_indentation/gtk_scintilla_get_tab_width(GTK_SCINTILLA(scintilla))));
-			}
-		}
-		else if (strcmp(member_function_buffer, "->")==0 && (gtk_scintilla_get_line_state(GTK_SCINTILLA(main_window.current_editor->scintilla), current_line)==274)) {
-			if (gtk_scintilla_autoc_active(GTK_SCINTILLA(scintilla))==1) {
-				autocomplete_member_function(scintilla, wordStart, wordEnd);
-			}
-			else {
-				if (completion_timer_set==FALSE) {
-					completion_timer_id = g_timeout_add(preferences.auto_complete_delay, auto_memberfunc_complete_callback, (gpointer) current_pos);
-					completion_timer_set=TRUE;
+				if (preferences.use_tabs_instead_spaces) {
+					gtk_scintilla_goto_pos(GTK_SCINTILLA(scintilla), gtk_scintilla_position_from_line(GTK_SCINTILLA(scintilla), current_line)+(previous_line_indentation/gtk_scintilla_get_tab_width(GTK_SCINTILLA(scintilla))));
+				}
+				else {
+					gtk_scintilla_goto_pos(GTK_SCINTILLA(scintilla), gtk_scintilla_position_from_line(GTK_SCINTILLA(scintilla), current_line)+(previous_line_indentation));
 				}
 			}
 		}
-		else if ((current_word_length>=3) && (gtk_scintilla_get_line_state(GTK_SCINTILLA(main_window.current_editor->scintilla), current_line)==274)) {
-			// check to see if they've typed <?php and if so do nothing
-			if (wordStart>1) {
-				ac_buffer = gtk_scintilla_get_text_range (GTK_SCINTILLA(scintilla), wordStart-2, wordEnd, &ac_length);
-				if (strcmp(ac_buffer,"<?php")==0) {
-					return;
+		else {
+			member_function_buffer = gtk_scintilla_get_text_range (GTK_SCINTILLA(scintilla), wordStart-2, wordStart, &member_function_length);
+			if (gtk_scintilla_call_tip_active(GTK_SCINTILLA(scintilla)) && ch==')') {
+				gtk_scintilla_call_tip_cancel(GTK_SCINTILLA(scintilla));
+			}
+			else if (type == TAB_PHP && (ch == '(') && 
+				(gtk_scintilla_get_line_state(GTK_SCINTILLA(scintilla), current_line))==274 &&
+				(calltip_timer_set==FALSE)) {
+					calltip_timer_id = g_timeout_add(preferences.calltip_delay, calltip_callback, (gpointer) current_pos);
+					calltip_timer_set=TRUE;
+			}
+			else if (type == TAB_PHP && strcmp(member_function_buffer, "->")==0 && 
+				(gtk_scintilla_get_line_state(GTK_SCINTILLA(scintilla), current_line)==274)) {
+				if (gtk_scintilla_autoc_active(GTK_SCINTILLA(scintilla))==1) {
+					autocomplete_member_function(scintilla, wordStart, wordEnd);
 				}
-			}
-
-			if (gtk_scintilla_autoc_active(GTK_SCINTILLA(scintilla))==1) {
-				autocomplete_word(scintilla, wordStart, wordEnd);
-			}
-			else {
-				completion_timer_id = g_timeout_add(preferences.auto_complete_delay, auto_complete_callback, (gpointer) current_pos);
-			}
-		}
-		// Drop down for HTML here (line_state = 272)
-	}
-	else if (main_window.current_editor->type == TAB_CSS) {
-		// Drop down if TAB_CSS here
-		if (ch=='\r' || ch=='\n') {
-			if (current_line>0) {
-				previous_line = current_line-1;
-				previous_line_indentation = gtk_scintilla_get_line_indentation(GTK_SCINTILLA(scintilla), previous_line);
-
-				if (1==1) {//preferences.auto_indent_after_brace) {
-					previous_line_end = gtk_scintilla_get_line_end_position(GTK_SCINTILLA(scintilla), previous_line);
-					previous_char_buffer = gtk_scintilla_get_text_range (GTK_SCINTILLA(scintilla), previous_line_end-1, previous_line_end, &previous_char_buffer_length);
-					if (*previous_char_buffer=='{') {
-						previous_line_indentation+=preferences.indentation_size;
+				else {
+					if (completion_timer_set==FALSE) {
+						completion_timer_id = g_timeout_add(preferences.auto_complete_delay, auto_memberfunc_complete_callback, (gpointer) current_pos);
+						completion_timer_set=TRUE;
 					}
 				}
-
-				indent_line(scintilla, current_line, previous_line_indentation);
-
-				//g_print("previous_line: %d\nprevious_indent: %d\n", previous_line, previous_line_indentation);
-				//gtk_scintilla_goto_pos(GTK_SCINTILLA(scintilla), gtk_scintilla_get_line_end_position(GTK_SCINTILLA(scintilla), current_line));
-				gtk_scintilla_goto_pos(GTK_SCINTILLA(scintilla), gtk_scintilla_position_from_line(GTK_SCINTILLA(scintilla), current_line)+(previous_line_indentation/gtk_scintilla_get_tab_width(GTK_SCINTILLA(scintilla))));
 			}
-		}
-		else if (current_word_length>=3) {
-			if (gtk_scintilla_autoc_active(GTK_SCINTILLA(scintilla))==1) {
-				css_autocomplete_word(scintilla, wordStart, wordEnd);
-			}
-			else {
-				completion_timer_id = g_timeout_add(preferences.auto_complete_delay, css_auto_complete_callback, (gpointer) current_pos);
-			}
-		}
-
-	}
-	else if (main_window.current_editor->type == TAB_SQL) {
-		// Drop down if TAB_SQL here
-		if (ch=='\r' || ch=='\n') {
-			if (current_line>0) {
-				previous_line = current_line-1;
-				previous_line_indentation = gtk_scintilla_get_line_indentation(GTK_SCINTILLA(scintilla), previous_line);
-
-				if (1==1) {//preferences.auto_indent_after_brace) {
-					previous_line_end = gtk_scintilla_get_line_end_position(GTK_SCINTILLA(scintilla), previous_line);
-					previous_char_buffer = gtk_scintilla_get_text_range (GTK_SCINTILLA(scintilla), previous_line_end-1, previous_line_end, &previous_char_buffer_length);
-					if (*previous_char_buffer=='{') {
-						previous_line_indentation+=preferences.indentation_size;
+			else if ((current_word_length>=3) && 
+				( (gtk_scintilla_get_line_state(GTK_SCINTILLA(scintilla), current_line)==274 && type == TAB_PHP) ||
+				  (type != TAB_HELP) )) {
+				// check to see if they've typed <?php and if so do nothing
+				if (wordStart>1) {
+					ac_buffer = gtk_scintilla_get_text_range (GTK_SCINTILLA(scintilla), wordStart-2, wordEnd, &ac_length);
+					if (strcmp(ac_buffer,"<?php")==0 && type == TAB_PHP) {
+						return;
 					}
 				}
-
-				indent_line(scintilla, current_line, previous_line_indentation);
-
-				//g_print("previous_line: %d\nprevious_indent: %d\n", previous_line, previous_line_indentation);
-				//gtk_scintilla_goto_pos(GTK_SCINTILLA(scintilla), gtk_scintilla_get_line_end_position(GTK_SCINTILLA(scintilla), current_line));
-				gtk_scintilla_goto_pos(GTK_SCINTILLA(scintilla), gtk_scintilla_position_from_line(GTK_SCINTILLA(scintilla), current_line)+(previous_line_indentation/gtk_scintilla_get_tab_width(GTK_SCINTILLA(scintilla))));
+	
+				if (gtk_scintilla_autoc_active(GTK_SCINTILLA(scintilla))==1) {
+					switch(type) {
+						case(TAB_PHP): autocomplete_word(scintilla, wordStart, wordEnd); break;
+						case(TAB_CSS): css_autocomplete_word(scintilla, wordStart, wordEnd); break;
+						case(TAB_SQL): sql_autocomplete_word(scintilla, wordStart, wordEnd); break;
+					}
+				}
+				else {
+					switch(type) {
+						case(TAB_PHP): completion_timer_id = g_timeout_add(preferences.auto_complete_delay, auto_complete_callback, (gpointer) current_pos); break;
+						case(TAB_CSS): completion_timer_id = g_timeout_add(preferences.auto_complete_delay, css_auto_complete_callback, (gpointer) current_pos); break;
+						case(TAB_SQL): completion_timer_id = g_timeout_add(preferences.auto_complete_delay, sql_auto_complete_callback, (gpointer) current_pos); break;
+					}
+					
+				}
 			}
+			// Drop down for HTML here (line_state = 272)
 		}
-		else if (current_word_length>=3) {
-			if (gtk_scintilla_autoc_active(GTK_SCINTILLA(scintilla))==1) {
-				sql_autocomplete_word(scintilla, wordStart, wordEnd);
-			}
-			else {
-				completion_timer_id = g_timeout_add(preferences.auto_complete_delay, sql_auto_complete_callback, (gpointer) current_pos);
-			}
-		}
-
 	}
 }
 
@@ -1533,7 +1643,7 @@ gchar *convert_to_full(gchar *filename)
 	gchar abs_buffer[2048];
 	GString *gstr_filename;
 	
-	if (g_strncasecmp(filename, "file://", MIN(strlen(filename), 7))==0) {
+	if (strstr(filename, "://") != NULL) {
 		return g_strdup(filename);	
 	}
 
